'use strict';

var useState = require('./useState');

module.exports = function createStore(reducer, initialState) {
  var state = initialState || {};
  var promisedState = state;
  var listeners = [];
  var store = {
    getState: function getState() {
      return state;
    },
    dispatch: function dispatch(action) {
      // for thunks
      if (typeof action === 'function') return action(store.dispatch, store.getState);
      // for promises
      if (Promise.resolve(action) === action) return Promise.resolve(action).then(store.dispatch);
      promisedState = Promise.resolve(promisedState).then(function (state) {
        return reducer(state, action || {});
      });
      return Promise.resolve(promisedState).then(function (result) {
        state = result;
        listeners.forEach(function (listener, i) {
          return listener(function () {
            delete listeners[i];
          });
        });
      });
    },
    subscribe: function subscribe(callback) {
      return listeners.push(callback);
    }
  };
  if (initialState) setImmediate(store.dispatch);

  // connect hooks to store
  useState.dispatch = store.dispatch;
  return store;
};